/* HAPI.H - HAPI User Functions and data structs
   Copyright (c) 1995 Sangoma Technologies Inc.
   Author: Glenn Robins
   API v1.0, 16-bit

   Application include file for HAPI.LIB functions.
   See HAPI.HLP for a complete function reference.

   Must #include <windows.h> before this header for type
   definitions used here.
*/

#define MDATALEN        1027
#define	CB_HEADERLEN	15

/* HIP control block */
typedef struct {
	char command;
	WORD buffer_length;
	char return_code;
	char PF_bit;
	char reserved[6];
	char time_stamp[4];
	char data[MDATALEN];
} CBLOCK;


/* HAPI will always return 1 (performs no function) */
int WINAPI HAPIAlive(void);

/* Initializes HAPI and loads SDLA driver, if not loaded already.
   A trace should be registered before this setup if one will
   be used at all, but is not necessary.
   Each application instance MUST call this on initialization.
   returns 0 if failed to load SDLA driver,
	   1 if this is the first setup call and the SDLA driver
	     loaded successfully,
	   2 if this is not the first successful setup call
	     (driver already loaded)
*/
int WINAPI HAPISetup(void);

/* Cleanup and unloads SDLA driver if this is the last
   application instance to use the driver.
   You must terminate your trace registration first if one
   was registered, unless you are the last application to use
   this DLL (but you shouldn't assume this).
   Each application instance MUST call this before closing.
   returns 1 always (return code reserved for future use)
*/
int WINAPI HAPICleanup(void);

/* returns non-zero if link in ABM */
int WINAPI HAPIqLinkInABM(BYTE cardnumber);

/* Emulate HIP interrupt call with control block
   returns return_code value in structure
*/
char WINAPI HAPIControl(CBLOCK far *block, BYTE cardnumber);

/* returns non-zero if trace data is available to be read by
   a READ_TRACE_DATA call
*/
char WINAPI HAPIqTraceDataAvailable(BYTE cardnumber);

/* returns number of frames pending for read
*/
unsigned char WINAPI HAPInDataToRead(BYTE cardnumber);

/* returns 1 if clear to send bit is set, 0 if not
*/
char WINAPI HAPIqClearToXmit(BYTE cardnumber);


/* returns 0 if a debugging command trace is already registered or
   had difficulty creating the window, 1 if successful.
   hWnd is a window to send notification messages to, and
   iMessage is a user-defined message that HAPI will send the
   application to signal an HAPI notification. The actual
   message code is passed by value to the WPARAM parameter
   of the window procedure
*/
char WINAPI HAPIRegisterDebug(HINSTANCE hInstance, HWND hWnd, UINT iMessage);

/* if a debugging command trace is active and instance matches that passed
   when registered (for security/fault tolerance purposes) then
   trace is terminated and returns 0. If instance doesn't match,
   returns 1. If no trace is running, returns 2.
   The trace is also terminated automatically if the window is
   closed (see trace messages)
*/
char WINAPI HAPITerminateDebug(HINSTANCE hInstance);

/* returns TRUE if a debugging command trace is registered and active */
BOOL WINAPI HAPIIsDebugRegistered(void);

/* DEBUGGING COMMAND TRACE MESSAGES (wParam values) */
#define	TRACEMSG_CLOSE	0

/*
   returns SDLAR_NOT_REGISTERED (0L) if failed to obtain a selector
	(any read or write calls to the board will result
	in a general protection fault)
   returns SDLAR_REGISTERED if succeeded (1L): the card's
	instance count was incremented but this is not the
	first registration call for this board (it is already
	in use by an application)
   returns SDLAR_ALLOCATED if succeeded (2L): the card's
	instance count was incremented and this is the
	first registration call for this board, and the
	memory selector was allocated
*/
int WINAPI HAPIRegisterCard(BYTE cardnumber);

/*
   returns SDLAR_NOT_REGISTERED (0L) if card wasn't registered
   returns SDLAR_DEREGISTERED (1L) if card's instance count
	was decremented but isn't currently zero
	(i.e. an application is still using the specified board)
   returns SDLAR_RELEASED (2L) if card is now not used by
	any application, and the selector was freed
*/
int WINAPI HAPIDeregisterCard(BYTE cardnumber);

#define SDLAR_NOT_REGISTERED    0
#define SDLAR_DEREGISTERED      1
#define SDLAR_RELEASED          2
#define SDLAR_REGISTERED        1
#define SDLAR_ALLOCATED         2

/*
   returns a bit record (32-bit) of cards registered for use
	and cards that exist:
	[ E15 E14 ... E3 E2 E1 E0 R15 R14 ... R3 R2 R1 R0 ]
   For example, if bit R2 is set, then card 2 is registered
		if bit E2 is set, then card 2 exists
			(i.e. a configuration entry exists
			in WIN.INI)
*/
LRESULT WINAPI HAPIGetCards(void);


