/*
   HBOUNCE.C - Sample program for HAPI.DLL
   Copyright (c) 1995 Sangoma Technologies Inc.
   Author: Glenn Robins
   March 1995

   Sample program for Windows 3.1 using the HAPI to bounce received
   frames back to the sender on the same logical card.

   Default logical card: 0
   To change this, create the file "HBOUNCE.INI" with the
   following entry:

   [bounce_parameters]
   logical_card=<card number (0-15)>

*/

/* for "portability" to Win32 in case we want to */
#define STRICT

#include <windows.h>
#include <windowsx.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef MAKEFORNT
#include "hapi32mc.h"
#else
#include "hapi.h"
#endif
#include "hapi_m.h"

#define SAYSTATE(x)     SetWindowText(exehwn, x)
#define SAYNOSTATE()    SetWindowText(exehwn, szProgramHeader)

/* dummy message sent to this window if HAPI Supervisor is closed */
#define MSG_DUMMY	0x4000
/* number of lines buffered in list box containing status messages */
#define COUNT_MAX       100

HINSTANCE ghinstance;
HWND exehwn, bbhwn, lbhwn, quhwn;

/* flag used to determine if first HDLC_WRITE succeeded or failed */
char rewrite=0;

char *szClassName = "HBOUNCEClass";
char *szTitleName = "The HBOUNCEr";
char szProgramHeader[40];
char *bouncebutton = "Bounce";
char *iniSection = "bounce_parameters";
char *ourinifile = "hbounce.ini";

BYTE card;	/* logical card number */

CBLOCK cb, auxb;	/* control blocks used to pass to HAPIControl */
unsigned long sent = 0;	/* currently updated but not reported */
LONG pause = 0;	/* number of main loop cycles to skip before next poll */

BOOL quitting_so_do_not_poll = TRUE;	/* don't call the board if TRUE */
BOOL dobounce = FALSE;	/* flag indicating whether bounce mode is on or off */

/* prototypes */
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmdShow);
LRESULT CALLBACK _export MainWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam );
void DoPolling(void);
void bounce(void);
void DoCommand(void);
void AddStringListBox(char far *m);
void copy_cb(CBLOCK *dest, CBLOCK *src);

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmdShow)
{
	MSG msg;
	BOOL fRetVal;
	WNDCLASS wndclass;
	int title_height;

	title_height=GetSystemMetrics(SM_CYCAPTION);
	ghinstance = hInstance;

	/* Don't allow more than one instance of this app */
	if (hPrevInstance != NULL) return(0);

	/* retrieve parameters if any */
	card = GetPrivateProfileInt(iniSection, "logical_card", 0, ourinifile);

	/* set up and register the window class */
	wndclass.style         = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc   = MainWndProc;
	wndclass.cbClsExtra    = 0;
	wndclass.cbWndExtra    = 0;
	wndclass.hInstance     = ghinstance;
	wndclass.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
	wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
	wndclass.hbrBackground = (HBRUSH)GetStockObject(GRAY_BRUSH);
	wndclass.lpszMenuName  = NULL;
	wndclass.lpszClassName = szClassName;

	if (!RegisterClass(&wndclass)) {
		return(0);
	}

	exehwn = CreateWindow( szClassName, szTitleName,
		WS_OVERLAPPED | WS_MINIMIZE | WS_MINIMIZEBOX | WS_CAPTION | WS_SYSMENU | WS_BORDER,
		CW_USEDEFAULT, 0, 350, title_height*8, HWND_DESKTOP, NULL, ghinstance, NULL);

	if (!exehwn)  return(0);	/* failed to create window */

	/* create controls (bounce button and status message list box) */
	bbhwn = CreateWindow("BUTTON", bouncebutton, WS_CHILD | WS_VISIBLE, 2, 2, 80, title_height, exehwn, NULL, ghinstance, NULL);
	quhwn = CreateWindow("BUTTON", "Quit", WS_CHILD | WS_VISIBLE, 266, 2, 80, title_height, exehwn, NULL, ghinstance, NULL);
	lbhwn = CreateWindow("LISTBOX", "", WS_CHILD | WS_VISIBLE | WS_VSCROLL | LBS_DISABLENOSCROLL, 1, title_height+5, 346,
		 title_height*6, exehwn, NULL, ghinstance, NULL);

	ShowWindow(exehwn, cmdShow);
	UpdateWindow(exehwn);

	/* register this instance for the use of the logical card */
	if (!HAPIRegisterCard(card)) {
		MessageBox(exehwn, "Failed to register for the use of the card!", "FATAL ERROR", MB_ICONHAND | MB_OK);
		DestroyWindow(exehwn);
		return(0);
	}

	sprintf(szProgramHeader, "%s (Card %u)", szTitleName, card);
	SAYNOSTATE();

	quitting_so_do_not_poll = FALSE;	/* allowed to poll now */
	fRetVal = TRUE;		/* keep looping until FALSE (WM_QUIT) */

	/* stay in this outside loop until the WM_QUIT message has
	been encountered and processed */
	while (fRetVal)  {
		/* if there are messages for us, process them until
		there are none left in our queue - then we do a
		polling call to check if this app needs to do anything */
		while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
			if (msg.message == WM_QUIT)
				fRetVal = FALSE;	/* leave main loop */
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		/* EXTREMELY IMPORTANT that you do not poll when the
		application does not have a card registered! */
		if (!quitting_so_do_not_poll)  DoPolling();
	}
	return fRetVal;
}

/* used to add a line of text to the list box for status messages */
void AddStringListBox(char far *m)
{
	static UINT count = 0;

	count++;
	if (count == COUNT_MAX)  {
		SendMessage(lbhwn, LB_DELETESTRING, 0, 0L);
		count--;
	}
	SendMessage(lbhwn, LB_ADDSTRING, 0, (LPARAM)m);
	SendMessage(lbhwn, LB_SETCURSEL, count-1, 0L);
}

/* main window procedure */
LRESULT CALLBACK _export MainWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam )
{
	HWND h;
	char s[80];

	switch(iMessage) {
	case WM_CREATE:
		/* on creation of the window, might as well
		register a debugging command trace and then
		set up the DLL for this app to use it */
		HAPIRegisterDebug(ghinstance, exehwn, MSG_DUMMY);
		HAPISetup();
		return(0);

	case WM_DESTROY:
		/* on closing of this app, set a flag to make sure
		no more SDLA access is attempted, deregister
		our app as a user of this logical card, terminate
		the debugging command trace, and tell the DLL
		that this app won't be calling it anymore */
		quitting_so_do_not_poll = TRUE;
		HAPIDeregisterCard(card);
		HAPITerminateDebug(ghinstance);
		HAPICleanup();
		PostQuitMessage(0);
		break;

	case WM_COMMAND:
		h = GET_WM_COMMAND_HWND(wParam, lParam);
		if (h == bbhwn) {
			/* if clicked the "bounce" button... */
			if (dobounce) {
				/* if bouncing was on, turn it off */
				dobounce = FALSE;
				SetWindowText(bbhwn, bouncebutton);
				SAYNOSTATE();
			} else {
				/* if bouncing was off, turn it on */
				SetWindowText(bbhwn, "STOP!");
				dobounce = TRUE;
				sprintf(s, "Bouncing (Card %u)", card);
				SAYSTATE(s);
			}
		}
		else if (h == quhwn) {
			/* if hit Quit button, let's quit nicely
			so that the cleanup code may run in the
			WM_DESTROY handler above */
			PostMessage(exehwn, WM_CLOSE, 0, 0L);
		}
		return(0);
	default:
		return DefWindowProc( hWnd, iMessage, wParam, lParam );
	}
	return(1);
}

/* generic command processing function
   the control block (CBLOCK)cb members must have already been set
   appropriately
*/
void DoCommand(void)
{
	BYTE result;
	char s[80], t[10];
	UINT n;

	/* pass the HIP/mailbox-compatible control block to the card
	via the HAPI.DLL and the SDLA driver */
	result = (BYTE)HAPIControl(&cb, card);

	/* if it successfully read a packet, display the first few
	bytes in the status list box */
	if (result == 0 && cb.command == HDLC_READ) {
		sprintf(s, ">>> ");
		for (n=0; n < cb.buffer_length; n++) {
			if (n > 5) { strcat(s, "..."); break; }
			sprintf(t, "%02X '%c' ", (BYTE)cb.data[n], cb.data[n]);
			strcat(s, t);
		}
		AddStringListBox(s);
	}

	/* indicate the command and return codes as a list box status line */
	sprintf(s, "Command: %02X  Return Code: %02X", cb.command, result);
	AddStringListBox(s);
}

/* essentially the main program loop outside the duty of processing
   window messages
*/
void DoPolling(void)
{
	static unsigned long poll_count = 0;

	/* let's interact with the SDLA card every other cycle so as
	to allow higher priority HAPI apps to run if they need to */
	if (pause > 0) { pause--; return; }
	pause = 1;

	/* currently not reported (may be used as a time reference) */
	poll_count++;

	/* now check if we are in bounce mode, and call the bounce
	handler if so */
	if (dobounce) bounce();
}

/* frame bouncer handler */
void bounce(void)
{
	/* check if we need to try resubmitting the last
	received frame for transmission if it failed last time */
	if (rewrite) {
		/* the last HDLC_WRITE attempt failed, so let's get
		a copy of the control block used last time and
		resubmit the request */
		copy_cb(&cb, &auxb);
		DoCommand();
		if (cb.return_code == 0) {
			/* the WRITE succeeded this time */
			rewrite=0; sent++;
		} else return;	/* the WRITE failed, so try next time */
	}

	/* this check is not functionally necessary, since this same
	check will be made on the HDLC_READ HAPIControl call below; however,
	this saves the HAPI Supervisor (the command trace) from having
	to report every HDLC_READ call whether anything was read or not. */
	if (!HAPInDataToRead(card)) return;

	/* there is something to read so read it */
	cb.command = HDLC_READ;
	cb.buffer_length=0;
	DoCommand();	/* perform command and display result */
	if (cb.return_code == 0) {
		/* the read was successful, so just change the command */
		cb.command = HDLC_WRITE;
		DoCommand();	/* attempt our first WRITE */
		if (cb.return_code != 0) {
			/* the first WRITE failed for some reason
			(maybe no buffers were available) - in any case,
			put it in our one-packet queue, ready for
			retransmission next time */
			rewrite=1;
			copy_cb(&auxb, &cb);
		}
		else { rewrite=0; sent++; }	/* it was sent! */
	}
}

void copy_cb(CBLOCK *dest, CBLOCK *src)
{
	memcpy((void *)dest, (void *)src, sizeof(CBLOCK));
}


